//
//  OJSCustomNavBarView.swift
//  OJASwiftKitDemo
//
//  Created by luxiaoming on 2016/12/5.
//  Copyright © 2016年 luxiaoming. All rights reserved.
//

import UIKit


public enum OJSCustomNavBarViewType {
    case type0_0           //左右都没有按钮，默认
    case type0_1           //左边没有，右边一个
    case type0_2           //左边没有，右边二个
    case type1_0           //左边一个，右边没有
    case type1_1           //左边一个，右边一个
    case type1_2           //左边一个，右边二个
    case type2_0           //左边二个，右边没有
    case type2_1           //左边二个，右边一个
    case type2_2           //左边二个，右边二个
}

private let kButtonWidth: CGFloat = 44
private let kButtonHeight: CGFloat = 44

public class OJSCustomNavBarView: UIView {

    public var titleLabel: UILabel!
    public var leftOneButton: UIButton!
    public var leftTwoButton: UIButton!
    public var rightOneButton: UIButton!
    public var rightTwoButton: UIButton!
    
    /// 默认是隐藏的，不受viewType影响，需手动设置
    public var bottomLineImageView: UIImageView!
    
    
    public var viewType: OJSCustomNavBarViewType = .type0_0 {
        didSet {
            switch viewType {
            case .type0_1:
                leftOneButton.isHidden = true
                leftTwoButton.isHidden = true
                rightOneButton.isHidden = false
                rightTwoButton.isHidden = true
            case .type0_2:
                leftOneButton.isHidden = true
                leftTwoButton.isHidden = true
                rightOneButton.isHidden = false
                rightTwoButton.isHidden = false
            case .type1_0:
                leftOneButton.isHidden = false
                leftTwoButton.isHidden = true
                rightOneButton.isHidden = true
                rightTwoButton.isHidden = true
            case .type1_1:
                leftOneButton.isHidden = false
                leftTwoButton.isHidden = true
                rightOneButton.isHidden = false
                rightTwoButton.isHidden = true
            case .type1_2:
                leftOneButton.isHidden = false
                leftTwoButton.isHidden = true
                rightOneButton.isHidden = false
                rightTwoButton.isHidden = false
            case .type2_0:
                leftOneButton.isHidden = false
                leftTwoButton.isHidden = false
                rightOneButton.isHidden = true
                rightTwoButton.isHidden = true
            case .type2_1:
                leftOneButton.isHidden = false
                leftTwoButton.isHidden = false
                rightOneButton.isHidden = false
                rightTwoButton.isHidden = true
            case .type2_2:
                leftOneButton.isHidden = false
                leftTwoButton.isHidden = false
                rightOneButton.isHidden = false
                rightTwoButton.isHidden = false
            default:
                leftOneButton.isHidden = true
                leftTwoButton.isHidden = true
                rightOneButton.isHidden = true
                rightTwoButton.isHidden = true
            }
        }
    }
    
    
    public init() {
        super.init(frame: CGRect(x: 0, y: 0, width: kOJSScreenWidth, height: 64))
        setupUI()
    }
    
    
    public required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
    }
    
    public override func awakeFromNib() {
        super.awakeFromNib()
        setupUI()
    }
}

// MARK: - PrivateMethod
private extension OJSCustomNavBarView {
    
    func setupUI() {
        self.backgroundColor = UIColor.clear
        titleLabel = UILabel(frame: CGRect(x: kButtonWidth * 2,
                                           y: 20,
                                           width: kOJSScreenWidth - kButtonWidth * 4,
                                           height: kButtonHeight))
        titleLabel.textAlignment = .center
        self.addSubview(titleLabel)
        
        leftOneButton = addDefaultNavBarButton(withFrame: CGRect(x: 0, y: 20, width: kButtonWidth, height: kButtonHeight))
        leftTwoButton = addDefaultNavBarButton(withFrame: CGRect(x: kButtonHeight, y: 20, width: kButtonWidth, height: kButtonHeight))
        rightOneButton = addDefaultNavBarButton(withFrame: CGRect(x: kOJSScreenWidth - kButtonWidth, y: 20, width: kButtonWidth, height: kButtonHeight))
        rightTwoButton = addDefaultNavBarButton(withFrame: CGRect(x: kOJSScreenWidth - kButtonWidth * 2, y: 20, width: kButtonWidth, height: kButtonHeight))
        
        bottomLineImageView = UIImageView(frame: CGRect(x: 0, y: 63.5, width: kOJSScreenWidth, height: 0.5))
        bottomLineImageView.backgroundColor = OJSColor(hexRGBValue: 0xffffff)
        bottomLineImageView.isHidden = true
        self.addSubview(bottomLineImageView)
        self.viewType = .type0_0
    }
    
    func addDefaultNavBarButton(withFrame frame: CGRect) -> UIButton {
        let button = UIButton(frame: frame)
        button.setTitleColor(UIColor.white, for: .normal)
        button.titleLabel?.font = UIFont.systemFont(ofSize: 12)
        self.addSubview(button)
        return button
    }
    
}


// MARK: - PublicMethod
extension OJSCustomNavBarView {
    func update(withTitle title: String?) {
        if let titleString = title {
            titleLabel.attributedText = NSAttributedString(string: titleString, attributes: UINavigationBar.appearance().titleTextAttributes)
        } else {
            titleLabel.text = nil
        }
    }
}
