//
//  OJSNavigationController.swift
//  OJASwiftKitDemo
//
//  Created by luxiaoming on 2016/11/19.
//  Copyright © 2016年 luxiaoming. All rights reserved.
//

import UIKit

open class OJSNavigationController: UINavigationController {
    
    /// 注意，如果取到的viewController不是OJSViewController就会返回nil
    open var topBaseViewController: OJSViewController? {
        return self.topViewController as? OJSViewController
    }
    
}


// MARK: - Lifecycle
extension OJSNavigationController {
    
    open override func viewDidLoad() {
        super.viewDidLoad()
        
    }
    
    open override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
}



extension OJSNavigationController {
    
    open override func pushViewController(_ viewController: UIViewController, animated: Bool) {
        if self.viewControllers.count > 0 {
            //这里push的时候默认都把hidesBottomBarWhenPushed设置为true了，目前还没遇见要设置为false的情况
            viewController.hidesBottomBarWhenPushed = true
        }
        if let viewController = self.topBaseViewController {
            if viewController.ojs_isPushOrPopAnimating {
                DLog("oh~骚年莫急~~controller push要一个个来~")
                return
            }
            viewController.ojs_isPushOrPopAnimating = true
        }
        super.pushViewController(viewController, animated: animated)
    }
    
    open override func popViewController(animated: Bool) -> UIViewController? {
        if let viewController = self.topBaseViewController {
            if viewController.ojs_isPushOrPopAnimating {
                DLog("oh~骚年莫急~~controller pop要一个个来~")
                return nil
            }
            viewController.ojs_isPushOrPopAnimating = true
        }
        return super.popViewController(animated: animated)
    }
    
}



// MARK: - StatusBar
extension OJSNavigationController {
    
    open override var preferredStatusBarStyle: UIStatusBarStyle {
        if let viewController = self.topViewController {
            return viewController.preferredStatusBarStyle
        } else {
            return .default
        }
    }
    
    open override var prefersStatusBarHidden: Bool {
        if let viewController = self.topViewController {
            return viewController.prefersStatusBarHidden
        } else {
            return false
        }
    }
}



// MARK: - Rotation
extension OJSNavigationController {
    //这里只需要返回self.topViewController的相应属性，让各个viewController自己控制就行了
    //present出来的viewController本来就是受自己控制，跟nav无关
    
    open override var shouldAutorotate: Bool {
        if let viewController = self.topViewController {
            return viewController.shouldAutorotate
        } else {
            return false
        }
    }
    
    open override var supportedInterfaceOrientations: UIInterfaceOrientationMask {
        if let viewController = self.topViewController {
            return viewController.supportedInterfaceOrientations
        }  else {
            return .portrait
        }
    }
    
    
    open override var preferredInterfaceOrientationForPresentation: UIInterfaceOrientation {
        if let viewController = self.topViewController {
            return viewController.preferredInterfaceOrientationForPresentation
        }  else {
            return .portrait
        }
    }
}
