//
//  OJSNibExtension.swift
//  OJASwiftKitDemo
//
//  Created by luxiaoming on 2016/11/14.
//  Copyright © 2016年 luxiaoming. All rights reserved.
//

import UIKit
import Foundation

extension NSObject {
    public class func ojs_absoluteClassName() -> String {
        let classString = NSStringFromClass(self.classForCoder())
        let array = classString.components(separatedBy: ".")
        let name = array.last!
        return name
    }
}

extension UICollectionReusableView {
    public static var ojs_reuseIdentifier: String {
        return self.ojs_absoluteClassName()
    }
}

extension UITableViewCell {
    public static var ojs_reuseIdentifier: String {
        return self.ojs_absoluteClassName()
    }
}

extension UIView {
    
    public class func ojs_nib() -> UINib {
        return UINib(nibName: self.ojs_absoluteClassName(), bundle: nil)
    }
    
    public class func ojs_loadFromNib() -> Self {
        return loadFromNibHelper()
    }
    
    
    private class func loadFromNibHelper<T>() -> T {
        let array = Bundle.main.loadNibNamed(self.ojs_absoluteClassName(), owner: nil, options: nil)
        return array?.last as! T
    }
    
}


extension UIViewController {
    
    /// 如果identifier与类名相同，则可以不传
    public class func ojs_loadFromStoryboard(storyboardName: String, identifier: String? = nil) -> Self {
        return loadFromStroyboardHelper(storyboardName: storyboardName, identifier: identifier)
    }
    
    
    private class func loadFromStroyboardHelper<T>(storyboardName: String, identifier: String?) -> T {
        let storyboard = UIStoryboard(name: storyboardName, bundle: nil)
        let temp = identifier ?? self.ojs_absoluteClassName()
        return storyboard.instantiateViewController(withIdentifier: temp) as! T
    }
}
