//
//  OJSOptionalExtension.swift
//  LXMBlockKit
//
//  Created by luxiaoming on 2017/9/22.
//

import UIKit

extension Optional where Wrapped: Collection {
    var isNilOrEmpty: Bool {
        switch self {
        case .some(let collection):
            return collection.isEmpty
        case .none:
            return true
        }
    }
    
    var ojs_count: Wrapped.IndexDistance {
        switch self {
        case .some(let collection):
            return collection.count
        case .none:
            return 0
        }
    }
}


extension RangeReplaceableCollection {
    
    public mutating func ojs_append(_ newElement: Self.Element?) {
        if let newElement = newElement {
            self.append(newElement)
        }
    }
    
    public mutating func ojs_append<S>(contentsOf newElements: S?) where S : Sequence, Self.Element == S.Element {
        if let newElements = newElements {
            self.append(contentsOf: newElements)
        }
    }
    
    public mutating func ojs_insert(_ newElement: Self.Element?, at i: Self.Index) {
        if let newElement = newElement {
            self.insert(newElement, at: i)
        }
    }
    
    public mutating func ojs_insert<S>(contentsOf newElements: S?, at i: Self.Index) where S : Collection, Self.Element == S.Element {
        if let newElements = newElements {
            self.insert(contentsOf: newElements, at: i)
        }
    }
}
